// =====================================================================================
//
//       Filename:  sql_lib.h
//
//    Description:  
//                     code edit with vim and set tw=2, default code is under utf-8.
//
//        Version:  $Id$
//        Created:  2020/9/8 10:26:34
//       Revision:  none
//       Compiler:  modern c compiler, test under vs2019.
//
//         Author:  lk (dd), lk@shucantech.com
//        Company:  SCT
//   Organization:  SCT-P
//
//        License:  SCT-CL V0
//
// =====================================================================================


#pragma once
#include <stdint.h>
#include <stdbool.h>


#define   VAL_SIZE_1 1  //VALUE SIZE 1
#define   VAL_SIZE_2 2  //VALUE SIZE 2
#define   VAL_SIZE_4 4  //VALUE SIZE 4
#define   VAL_SIZE_8 8  //VALUE SIZE 8
#define   VAL_SIGN   0  //VALUE SIGNED
#define   VAL_USGN   1  //VALUE UNSIGNED

#define   VAL_TYPE_INV -1  //VALUE TYPE INVALID
#define   VAL_TYPE_I 0    //VALUE TYPE INTEGER
#define   VAL_TYPE_F 1    //VALUE TYPE FLOAT
#define   VAL_TYPE_S 2    //VALUE TYPE STRING
#define   VAL_TYPE_D 3    //VALUE TYPE DATE
#define   VAL_TYPE_T 4    //VALUE TYPE TIME
#define   VAL_TYPE_DT 5    //VALUE TYPE DATETIME

#ifdef _WIN32
#ifdef SQL_LIB_EXPORT
#define dllemport __declspec(dllexport)
#else
#define dllemport __declspec(dllimport)
#endif
#else
#define dllemport
#endif


#ifdef __cplusplus
extern "C" {
#endif

// ===  FUNCTION  ======================================================================
//         Name:  connect_to_server
//  Description:  connect to remote server
//                ip:    server's ip
//                port: server's port
//                user: user
//                pwd:  password
//
//                return value:   if connect and validated return true, else return false.
// =====================================================================================
dllemport bool connect_to_server(const char* ip, int port, const char* user, const char* pwd);


// ===  FUNCTION  ======================================================================
//         Name:  exit_to_server
//  Description:  exit to server
//
//                return value:   if exit succesful return true, else return false.
// =====================================================================================
dllemport bool exit_to_server();

// ===  FUNCTION  ======================================================================
//         Name:  exec
//  Description:  execute sql in server
//
//                sql:  sql's string
//                c  :  sql's size
//                return value:   if execute succesful on server return dataset's pointer, else return NULL.
// =====================================================================================
dllemport void* exec(const char* sql, int c);

// ===  FUNCTION  ======================================================================
//         Name:  del_datas
//  Description:  delete dataset executed.
//
//                d:    dataset's pointer
// =====================================================================================
dllemport void del_datas(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  rows_of
//  Description:  get the row's count of dataset
//
//                d:    dataset's pointer
//                return value:   if the d is NULL return 0, else return valid row's count.
// =====================================================================================
dllemport uint64_t rows_of(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  cols_of
//  Description:  get the column's count of dataset
//
//                d:    dataset's pointer
//                return value:   if the d is NULL return 0, else return valid column's count.
// =====================================================================================
dllemport uint64_t cols_of(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  sid_of
//  Description:  get the internal's id(same result as function 'id()')
//
//                d:    dataset's pointer
//                r:    dataset's row index
//                return value:   reutrn the system's id, SIZEMAX is mean id is generated and cant edit.
// =====================================================================================
dllemport uint64_t sid_of(void* d, uint64_t r);

// ===  FUNCTION  ======================================================================
//         Name:  col_name
//  Description:  get the column'a name of dataset
//
//                col_sz:    the name's length(pointer) of column.
//                d:        dataset's pointer
//                col_idx:  column index(start from 0)
//                return value:        if d is NULL or column's count is 0 return NULL, else return column's name,
//                              col_sz is name's length, note:if the d is deleted the return is invalid pointer.
// =====================================================================================
dllemport const char* col_name(int* col_sz, void* d, int col_idx);

// ===  FUNCTION  ======================================================================
//         Name:  at
//  Description:  get the cell's value.
//
//                row:    row index of dataset(start from 0)
//                col:    column index of dataset(start from 0)
//                d:      dataset's pointer
//                return value:      if d is NULL or row or col is invalid, return NULL, 
//                            else return the cell's pointer, note:if the d is deleted the return is invalid pointer.
// =====================================================================================
dllemport void* at(uint64_t row, uint64_t col, void* d);

// ===  FUNCTION  ======================================================================
//         Name:  print
//  Description:  print the dataset on stdout
//
//                d:      dataset's pointer
// =====================================================================================
dllemport void print(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  val_size
//  Description:  get the value's size.
//  
//                v:      value's pointer
//                return value:      if v is NULL or is string return NULL,  else return the value's size(bytes) 
//                            size is defined as VAL_SIZE_X
// =====================================================================================
dllemport int val_size(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  val_type
//  Description:  get the value's type.
//  
//                v:      value's pointer
//                return value:      if v is NULL return VAL_TYPE_INV,  else return the type 
//                            type is defined as VAL_TYPE_X
// =====================================================================================
dllemport int val_type(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  val_sign
//  Description:  get the value's is signed or sign.
//  
//                v:      value's pointer
//                return value:      if the value is signed integer return VAL_SIGN,  else return VAL_USGN
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport int val_sign(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_int8
//  Description:  convert value's pointer to type int8
//  
//                v:      value's pointer
//                return value:      if the value is type int8 return itself, else as static_cast result.
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport int8_t as_int8(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_int16
//  Description:  convert value's pointer to type int16
//  
//                v:      value's pointer
//                return value:      if the value is type int16 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport int16_t as_int16(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_int32
//  Description:  convert value's pointer to type int32
//  
//                v:      value's pointer
//                return value:      if the value is type int32 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport int32_t as_int32(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_int64
//  Description:  convert value's pointer to type int64
//  
//                v:      value's pointer
//                return value:      if the value is type int64 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport int64_t as_int64(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_uint8
//  Description:  convert value's pointer to type uint8
//  
//                v:      value's pointer
//                return value:      if the value is type uint8 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport uint8_t as_uint8(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_uint16
//  Description:  convert value's pointer to type uint16
//  
//                v:      value's pointer
//                return value:      if the value is type uint16 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport uint16_t as_uint16(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_uint32
//  Description:  convert value's pointer to type uint32
//  
//                v:      value's pointer
//                return value:      if the value is type uint32 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport uint32_t as_uint32(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_uint64
//  Description:  convert value's pointer to type uint64
//  
//                v:      value's pointer
//                return value:      if the value is type uint64 return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport uint64_t as_uint64(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_float
//  Description:  const value's pointer to type float
//  
//                v:      value's pointer
//                return value:      if the value is type float return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport float as_float(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_double
//  Description:  convert value's pointer to type double
//  
//                v:      value's pointer
//                return value:      if the value is type double return itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport double as_double(void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_str
//  Description:  convert value's pointer to type string
//  
//                b:      string's pointer
//                c:      string's length's pointer.
//                v:      value's pointer
//                return value:      if the value is type string *b is itself, else as static_cast result
//                            note: if the v is NULL will be exception.
// =====================================================================================
dllemport void as_str(char** b, uint64_t* c, void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_date
//  Description:  convert value's pointer to type date
//  
//                y:      year's pointer
//                m:      month's pointer
//                d:      day's pointer
//                v:      value's pointer
//                return value:      if v is type datetime/date or string can converted return converted date, else will be exception.
// =====================================================================================
dllemport void as_date(int* y, int* m, int* d, void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_time
//  Description:  convert value's pointer to type time
//  
//                h:      hour's pointer
//                m:      minute's pointer
//                s:      second's pointer
//                n:      nanosecond's pointer
//                v:      value's pointer
//                return value:      if v is type datetime/time or string can converted return converted time, else will be exception
// =====================================================================================
dllemport void as_time(int* h, int* m, int* s, int* n, void* v);

// ===  FUNCTION  ======================================================================
//         Name:  as_datetime
//  Description:  convert value's pointer to type datetime
//  
//                y:      year's pointer
//                m:      month's pointer
//                d:      day's pointer
//                h:      hour's pointer
//                mi:      minute's pointer
//                s:      second's pointer
//                n:      nanosecond's pointer
//                v:      value's pointer
//                return value:      if v is type datetime or string can converted return converted datetime, else will be exception
// =====================================================================================
dllemport void as_datetime(int* y, int* m, int* d, int* h, int* mi, int* s, int* n, void* v);

// ===  FUNCTION  ======================================================================
//         Name:  can_update
//  Description:  give the info of the select statement is can update.
//                    some express or post operator will occur the dataset is can map to original.
//  
//                sql:      sql's pointer
//                sql_sz:   sql's size
//                return value:      return true if sql is can update, else return false
// =====================================================================================
dllemport bool can_update(const char* sql, int sql_sz);

// ===  FUNCTION  ======================================================================
//         Name:  extract_where_from_update
//  Description:  extract the where's statement from the update statement.
//                   
//  
//                sql:      sql's pointer
//                sql_sz:   sql's size
//                wsz:      size of where's string
//                return value:      return where's begin pointer
// =====================================================================================
dllemport const char* extract_where_from_update(const char* sql, int sql_sz, int* wsz);

// ===  FUNCTION  ======================================================================
//         Name:  extract_tab_name_from_update
//  Description:  give the table's name from the update statement.
//                    
//  
//                sql:      sql's pointer
//                sql_sz:   sql's size
//                tsz:      size of table's name.
//                return value:      return where's begin pointer
// =====================================================================================
dllemport const char* extract_tab_name_from_update(const char* sql, int sql_sz, int* tsz);


#ifdef __cplusplus
}
#endif

